import psycopg2
from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from environ import Env
from psycopg2.extensions import ISOLATION_LEVEL_AUTOCOMMIT


class PsycopgBaseCommand(BaseCommand):
    help = 'Clean database (vacuum full and reindex)'

    def add_arguments(self, parser):
        parser.add_argument(
            '--db-url',
            dest='db_url',
            help='Connection string to Database',
        )

    def _create_connection(self, db_object):
        if db_object['ENGINE'] not in {'django.db.backends.postgresql_psycopg2',
                                       'django.db.backends.postgresql',
                                       'django.contrib.gis.db.backends.postgis'}:
            raise CommandError('The supplied DB object targets unsupported database backend!')

        con = psycopg2.connect(
            "host='{HOST}' dbname='{NAME}' user='{MASTER_USER}' password='{MASTER_PASSWORD}' port='{PORT}'"
            .format(**db_object)
        )

        # CREATE/DROP commands cannot run in transaction ==> turning it off for the connection
        con.set_isolation_level(ISOLATION_LEVEL_AUTOCOMMIT)
        return con

    def _get_db_object(self, **options):
        if options['db_url']:
            db_object = Env.db_url_config(options['db_url'])
        else:
            db_object = settings.DATABASES['default']
        db_object['MASTER_USER'] = db_object['USER']
        db_object['MASTER_PASSWORD'] = db_object['PASSWORD']
        return db_object



from psycopg2_extension.base_command import PsycopgBaseCommand


class Command(PsycopgBaseCommand):
    help = 'Clean database (vacuum and reindex)'

    def handle(self, *args, **options):
        db_object = self._get_db_object(**options)
        conn = self._create_connection(db_object)

        with conn.cursor() as cursor:
            vacuum_query = 'VACUUM ANALYZE'
            self.stdout.write('Running "{}"'.format(vacuum_query))
            cursor.execute(vacuum_query)
            reindex_query = 'REINDEX DATABASE {}'.format(db_object['NAME'])
            self.stdout.write('Running "{}"'.format(reindex_query))
            cursor.execute(reindex_query)



from setuptools import setup, find_packages

setup(
    name="django-psycopg2-extension",
    version='0.0.3',
    description="",
    author='Lubos Matl',
    author_email='matllubos@gmail.com',
    url='https://github.com/druids/django-psycopg2-extension',
    license='MIT',
    package_dir={'psycopg2_extension': 'psycopg2_extension'},
    include_package_data=True,
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
    ],
    install_requires=[
        'django>=1.11',
        'django-environ>=0.4.5',
    ],
    zip_safe=False
)



from psycopg2 import sql

from django.conf import settings

from psycopg2_extension.base_command import PsycopgBaseCommand


class Command(PsycopgBaseCommand):
    help = 'Initialize the database. It is necessary to use role which has permissions to drop & create databases ' \
           'and roles.'

    def handle(self, *args, **options):
        db_object = self._get_db_object(**options)
        db_name = db_object['NAME']
        db_object['NAME'] = 'template1'
        conn = self._create_connection(db_object)

        with conn.cursor() as cursor:
            cursor.execute("SELECT pid, pg_terminate_backend(pid) "
                           "FROM pg_stat_activity "
                           "WHERE pid <> pg_backend_pid() AND datname = %s", [db_name])
            cursor.execute(sql.SQL("DROP DATABASE IF EXISTS {}").format(sql.Identifier(db_name)))
            cursor.execute(sql.SQL("CREATE DATABASE {}").format(sql.Identifier(db_name)))
            for extension in getattr(settings, 'PSYCOPG2_EXTENSIONS', ()):
                cursor.execute(sql.SQL("CREATE EXTENSION IF NOT EXISTS {}".format(extension)))
